@@ -17,13 +17,15 @@
  * under the License.
  */
 
-var log = require('./log');
 var Int64 = require('node-int64');
 var InputBufferUnderrunError = require('./transport').InputBufferUnderrunError;
 var Thrift = require('./thrift');
 var Type = Thrift.Type;
 var util = require("util");
 
+var Int64Util = require('./int64_util');
+var json_parse = require('./json_parse');
+
 var InputBufferUnderrunError = require('./input_buffer_underrun_error');
 
 module.exports = TJSONProtocol;
@@ -89,9 +91,16 @@ TJSONProtocol.RType.set = Type.SET;
 TJSONProtocol.Version = 1;
 
 TJSONProtocol.prototype.flush = function() {
+  this.writeToTransportIfStackIsFlushable();
   return this.trans.flush();
 };
 
+TJSONProtocol.prototype.writeToTransportIfStackIsFlushable = function() {
+  if (this.tstack.length === 1) {
+    this.trans.write(this.tstack.pop());
+  }
+};
+
 /**
  * Serializes the beginning of a Thrift RPC message.
  * @param {string} name - The service method to call.
@@ -99,9 +108,6 @@ TJSONProtocol.prototype.flush = function() {
  * @param {number} seqid - The sequence number of this call (always 0 in Apache Thrift).
  */
 TJSONProtocol.prototype.writeMessageBegin = function(name, messageType, seqid) {
-  this.tstack = [];
-  this.tpos = [];
-
   this.tstack.push([TJSONProtocol.Version, '"' + name + '"', messageType, seqid]);
 };
 
@@ -116,6 +122,7 @@ TJSONProtocol.prototype.writeMessageEnd = function() {
 
   this.wbuf = '[' + this.wobj.join(',') + ']';
 
+  // we assume there is nothing more to come so we write
   this.trans.write(this.wbuf);
 };
 
@@ -148,6 +155,8 @@ TJSONProtocol.prototype.writeStructEnd = function() {
 
   str += '}';
   this.tstack[p] = str;
+
+  this.writeToTransportIfStackIsFlushable();
 };
 
 /**
@@ -178,6 +187,8 @@ TJSONProtocol.prototype.writeFieldEnd = function() {
       fieldInfo.fieldType + ':' + value + '}';
   }
   this.tpos.pop();
+
+  this.writeToTransportIfStackIsFlushable();
 };
 
 /**
@@ -234,6 +245,8 @@ TJSONProtocol.prototype.writeMapEnd = function() {
 
   this.tstack[p].push(map);
   this.tstack[p] = '[' + this.tstack[p].join(',') + ']';
+
+  this.writeToTransportIfStackIsFlushable();
 };
 
 /**
@@ -259,6 +272,8 @@ TJSONProtocol.prototype.writeListEnd = function() {
   }
 
   this.tstack[p] = '[' + this.tstack[p].join(',') + ']';
+
+  this.writeToTransportIfStackIsFlushable();
 };
 
 /**
@@ -284,6 +299,8 @@ TJSONProtocol.prototype.writeSetEnd = function() {
   }
 
   this.tstack[p] = '[' + this.tstack[p].join(',') + ']';
+
+  this.writeToTransportIfStackIsFlushable();
 };
 
 /** Serializes a boolean */
@@ -308,7 +325,11 @@ TJSONProtocol.prototype.writeI32 = function(i32) {
 
 /** Serializes a number */
 TJSONProtocol.prototype.writeI64 = function(i64) {
-  this.tstack.push(i64);
+  if (i64 instanceof Int64) {
+    this.tstack.push(Int64Util.toDecimalString(i64));
+  } else {
+    this.tstack.push(i64);
+  }
 };
 
 /** Serializes a number */
@@ -317,11 +338,19 @@ TJSONProtocol.prototype.writeDouble = function(dub) {
 };
 
 /** Serializes a string */
-TJSONProtocol.prototype.writeString = function(str) {
+TJSONProtocol.prototype.writeString = function(arg) {
   // We do not encode uri components for wire transfer:
-  if (str === null) {
+  if (arg === null) {
       this.tstack.push(null);
   } else {
+      if (typeof arg === 'string') {
+        var str = arg;
+      } else if (arg instanceof Buffer) {
+        var str = arg.toString('utf8');
+      } else {
+        throw new Error('writeString called without a string/Buffer argument: ' + arg);
+      }
+
       // concat may be slower than building a byte buffer
       var escapedString = '';
       for (var i = 0; i < str.length; i++) {
@@ -358,7 +387,15 @@ TJSONProtocol.prototype.writeString = function(str) {
 
 /** Serializes a string */
 TJSONProtocol.prototype.writeBinary = function(arg) {
-  this.writeString(arg);
+  if (typeof arg === 'string') {
+    var buf = new Buffer(arg, 'binary');
+  } else if (arg instanceof Buffer ||
+             Object.prototype.toString.call(arg) == '[object Uint8Array]')  {
+    var buf = arg;
+  } else {
+    throw new Error('writeBinary called without a string/Buffer argument: ' + arg);
+  }
+  this.tstack.push('"' + buf.toString('base64') + '"');
 };
 
 /**
@@ -418,17 +455,18 @@ TJSONProtocol.prototype.readMessageBegin = function() {
   }
 
   if (openBracketCount !== 0) {
-    throw new Error("Malformed JSON input, mismatched backets");
+    // Missing closing bracket. Can be buffer underrun.
+    throw new InputBufferUnderrunError();
   }
 
   //Reconstitute the JSON object and conume the necessary bytes
-  this.robj = JSON.parse(transBuf.buf.slice(transBuf.readIndex, cursor+1));
+  this.robj = json_parse(transBuf.buf.slice(transBuf.readIndex, cursor+1).toString());
   this.trans.consume(cursor + 1 - transBuf.readIndex);
 
   //Verify the protocol version
   var version = this.robj.shift();
   if (version != TJSONProtocol.Version) {
-    throw 'Wrong thrift protocol version: ' + version;
+    throw new Error('Wrong thrift protocol version: ' + version);
   }
 
   //Objectify the thrift message {name/type/sequence-number} for return
@@ -595,7 +633,12 @@ TJSONProtocol.prototype.readListBegin = function() {
 
 /** Deserializes the end of a list. */
 TJSONProtocol.prototype.readListEnd = function() {
-  this.readFieldEnd();
+  var pos = this.rpos.pop() - 2;
+  var st = this.rstack;
+  st.pop();
+  if (st instanceof Array && st.length > pos && st[pos].length > 0) {
+    st.push(st[pos].shift());
+  }
 };
 
 /**
@@ -611,28 +654,24 @@ TJSONProtocol.prototype.readSetEnd = function() {
   return this.readListEnd();
 };
 
-/** Returns an object with a value property set to
- *  False unless the next number in the protocol buffer
- *  is 1, in which case the value property is True */
 TJSONProtocol.prototype.readBool = function() {
-  return this.readI32() == '1';
+  return this.readValue() == '1';
 };
 
-/** Returns the an object with a value property set to the
-    next value found in the protocol buffer */
 TJSONProtocol.prototype.readByte = function() {
   return this.readI32();
 };
 
-/** Returns the an object with a value property set to the
-    next value found in the protocol buffer */
 TJSONProtocol.prototype.readI16 = function() {
   return this.readI32();
 };
 
-/** Returns the an object with a value property set to the
-    next value found in the protocol buffer */
 TJSONProtocol.prototype.readI32 = function(f) {
+  return +this.readValue();
+}
+
+/** Returns the next value found in the protocol buffer */
+TJSONProtocol.prototype.readValue = function(f) {
   if (f === undefined) {
     f = this.rstack[this.rstack.length - 1];
   }
@@ -645,7 +684,7 @@ TJSONProtocol.prototype.readI32 = function(f) {
     } else {
       r.value = f.shift();
     }
-  } else if (f instanceof Object) {
+  } else if (!(f instanceof Int64) && f instanceof Object) {
     for (var i in f) {
       if (i === null) {
         continue;
@@ -664,28 +703,26 @@ TJSONProtocol.prototype.readI32 = function(f) {
   return r.value;
 };
 
-/** Returns the an object with a value property set to the
-    next value found in the protocol buffer */
 TJSONProtocol.prototype.readI64 = function() {
-  return new Int64(this.readI32());
+  var n = this.readValue()
+  if (typeof n === 'string') {
+    // Assuming no one is sending in 1.11111e+33 format
+    return Int64Util.fromDecimalString(n);
+  } else {
+    return new Int64(n);
+  }
 };
 
-/** Returns the an object with a value property set to the
-    next value found in the protocol buffer */
 TJSONProtocol.prototype.readDouble = function() {
   return this.readI32();
 };
 
-/** Returns the an object with a value property set to the
-    next value found in the protocol buffer */
 TJSONProtocol.prototype.readBinary = function() {
-  return this.readString();
+  return new Buffer(this.readValue(), 'base64');
 };
 
-/** Returns the an object with a value property set to the
-    next value found in the protocol buffer */
 TJSONProtocol.prototype.readString = function() {
-  return this.readI32();
+  return this.readValue();
 };
 
 /**
@@ -701,5 +738,5 @@ TJSONProtocol.prototype.getTransport = function() {
  * Method to arbitrarily skip over data
  */
 TJSONProtocol.prototype.skip = function(type) {
-  throw 'skip not supported yet';
+  throw new Error('skip not supported yet');
 };
