@@ -23,8 +23,6 @@
 
 namespace Thrift\Exception;
 
-use Thrift\Exception\TException;
-
 /**
  * Protocol module. Contains all the types and definitions needed to implement
  * a protocol encoder/decoder.
@@ -35,7 +33,8 @@ use Thrift\Exception\TException;
 /**
  * Protocol exceptions
  */
-class TProtocolException extends TException {
+class TProtocolException extends TException
+{
   const UNKNOWN = 0;
   const INVALID_DATA = 1;
   const NEGATIVE_SIZE = 2;
@@ -44,7 +43,8 @@ class TProtocolException extends TException {
   const NOT_IMPLEMENTED = 5;
   const DEPTH_LIMIT = 6;
 
-  function __construct($message=null, $code=0) {
+  public function __construct($message=null, $code=0)
+  {
     parent::__construct($message, $code);
   }
 }
